@@ -30,6 +30,8 @@ class Simulation(object):
             "before_start": None,
             "before_step": None,
             "after_step": None,
+            "before_report": None,
+            "after_report": None,
             "before_end": None,
             "after_end": None,
             "after_close": None
@@ -75,8 +77,10 @@ class Simulation(object):
 
     def __exit__(self, *a):
         if self._isStarted:
+            self._model.simuwater_report()
             self._model.simuwater_end()
             self._isStarted = False
+            self._execute_callback(self.after_report())
             self._execute_callback(self.after_end())
         if self._isOpen:
             self._model.simuwater_close()
@@ -134,6 +138,16 @@ class Simulation(object):
 
     def after_end(self):
         return self._callbacks["after_end"]
+    
+    def before_report(self):
+        return self._callbacks["before_report"]
+    
+    def add_before_report(self, callback):
+        if self._is_callback(callback):
+            self._callbacks["before_report"] = callback
+    
+    def after_report(self):
+        return self._callbacks["after_report"]
 
     def add_after_end(self, callback):
         if self._is_callback(callback):
